(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[      6957,        273]
NotebookOptionsPosition[      5104,        207]
NotebookOutlinePosition[      6097,        237]
CellTagsIndexPosition[      6054,        234]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["PFAFFIAN PACKAGE SYMBOL", "PacletNameCell"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell["Pf", "ObjectName",
 CellID->3706],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{
       TemplateBox[{Cell[
          TextData["Pf"]],"paclet:ref/Det"},
        "RefLink",
        BaseStyle->"InlineFormula"], "[", 
       StyleBox["m", "TI"], "]"}]], "InlineFormula"],
     "\[LineSeparator]gives the pfaffian of the screw symmetric matrix ",
     Cell[BoxData[
      StyleBox["m", "TI"]], "InlineFormula"],
     ". "
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}],

Cell[CellGroupData[{

Cell[TextData[{
 "Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "NotesSection",
 CellID->48364535],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 Editable->True,
 WholeCellGroupOpener->True,
 CellID->881371396],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<Pfaffian`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->623148243],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pf", "[", 
  TagBox[
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {"0", "a"},
      {
       RowBox[{"-", "a"}], "0"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.7]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
   Function[BoxForm`e$, 
    MatrixForm[BoxForm`e$]]], "]"}]], "Input",
 Editable->True,
 CellLabel->"In[2]:=",
 CellID->228759878],

Cell[BoxData["a"], "Output",
 Editable->True,
 ImageSize->{123, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->643722991]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pf", "[", 
  RowBox[{"(", GridBox[{
     {"0", "a", "b", "c"},
     {
      RowBox[{"-", "a"}], "0", "d", "e"},
     {
      RowBox[{"-", "b"}], 
      RowBox[{"-", "d"}], "0", "f"},
     {
      RowBox[{"-", "c"}], 
      RowBox[{"-", "e"}], 
      RowBox[{"-", "f"}], "0"}
    }], ")"}], "]"}]], "Input",
 Editable->True,
 CellLabel->"In[3]:=",
 CellID->659410434],

Cell[BoxData[
 RowBox[{
  RowBox[{"c", " ", "d"}], "-", 
  RowBox[{"b", " ", "e"}], "+", 
  RowBox[{"a", " ", "f"}]}]], "Output",
 Editable->True,
 ImageSize->{527, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->29696151]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pf", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", 
      RowBox[{"-", "1"}], ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "0", ",", 
      RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "}"}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->480188104],

Cell[BoxData["2"], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->887003753]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Pf", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"#", "-", 
       RowBox[{"Transpose", "[", "#", "]"}]}], ")"}], "&"}], "@", 
    RowBox[{"RandomReal", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"200", ",", "200"}], "}"}]}], "]"}]}], "]"}], "]"}]], "Input",
 Editable->True,
 CellLabel->"In[5]:=",
 CellID->47106202],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.14646899999999998809308010550012113526`5.186345629754281", ",", 
   RowBox[{"-", "5.111850412971802`*^83"}]}], "}"}]], "Output",
 Editable->True,
 ImageSize->{205, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->40599197]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{477, 650},
WindowMargins->{{30, Automatic}, {Automatic, 3}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
ShowSelection->True,
TaggingRules->{"LinkTrails" -> "", "SearchTextTranslated" -> ""},
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 20, \
2012)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{$RootDirectory, "Applications", "Mathematica 9.app", 
        "SystemFiles", "FrontEnd", "StyleSheets", "Wolfram"}, "Reference.nb", 
       CharacterEncoding -> "UTF-8"]]]}, WindowSize -> {740, 671}, 
  WindowMargins -> {{44, Automatic}, {-36, Automatic}}, Visible -> False, 
  FrontEndVersion -> 
  "9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 20, 2012)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 113, 3, 39, "AnchorBarGrid",
 CellID->1],
Cell[673, 25, 39, 1, 43, "ObjectName",
 CellID->3706],
Cell[715, 28, 578, 18, 84, "Usage"],
Cell[CellGroupData[{
Cell[1318, 50, 139, 5, 29, "NotesSection",
 CellID->48364535],
Cell[CellGroupData[{
Cell[1482, 59, 212, 7, 24, "ExampleSection",
 CellID->881371396],
Cell[1697, 68, 117, 3, 25, "Input",
 CellID->623148243],
Cell[CellGroupData[{
Cell[1839, 75, 710, 22, 42, "Input",
 CellID->228759878],
Cell[2552, 99, 176, 6, 34, "Output",
 CellID->643722991]
}, Open  ]],
Cell[CellGroupData[{
Cell[2765, 110, 390, 16, 74, "Input",
 CellID->659410434],
Cell[3158, 128, 276, 10, 34, "Output",
 CellID->29696151]
}, Open  ]],
Cell[CellGroupData[{
Cell[3471, 143, 537, 17, 39, "Input",
 CellID->480188104],
Cell[4011, 162, 208, 7, 34, "Output",
 CellID->887003753]
}, Open  ]],
Cell[CellGroupData[{
Cell[4256, 174, 488, 17, 40, "Input",
 CellID->47106202],
Cell[4747, 193, 317, 9, 39, "Output",
 CellID->40599197]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
